/***************************************************************************
                          sqlguidoc.h  -  description
                             -------------------
    begin                : Sat Apr 14 18:28:06 CEST 2001
    copyright            : (C) 2001 by mb
    email                : marcus@beranek.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef SQLGUIDOC_H
#define SQLGUIDOC_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif 

// include files for QT
#include <qobject.h>
#include <qstring.h>
#include <qlist.h>

// include files for KDE
#include <kurl.h>

#include <string.h>
#include <stdlib.h>

#include <mysql.h>

// forward declaration of the SqlGui classes
class SqlGuiView;


//
// Structs fr MySQL-Daten, -Ergebnisse, -Tabellen etc.
//
// Liste der Tabellen in einer DB:

// Liste der Spalten einer Tabelle;
typedef
struct spalte{
	const char * name;
	char * type;	 									// Feldtyp
	char pri;									// Lngenangabe zum Typ
	const char * extra_flags; 	// fr INDEX, NOT NULL, AUTO_INC etc.
	struct spalte * next;
} spalten_t;


// Liste der Tabellen einer Datenbank:
typedef
struct table{
	const char * name;
	int type;
	const char * type_n;
	spalten_t * spalten_liste;
	struct table * next;
} table_t;


// Liste der Datenbanken:
typedef
struct base{
	const char * name;
	table_t * table_list;
	struct base * next;
} base_t;


// Darstellung der SQL-Abfrage in Reihen (row) und Spalten (field):
//
// Spalte:
typedef
struct field{
	int length;
	char * value;
	struct field * next;
} field_t;

// Zeile:
typedef
struct row{
	field_t * feld;
	struct row * next;
} row_t;

/**	SqlGuiDoc provides a document object for a document-view model.
  *
  * The SqlGuiDoc class provides a document object that can be used in conjunction with the classes SqlGuiApp and SqlGuiView
  * to create a document-view model for standard KDE applications based on KApplication and KMainWindow. Thereby, the document object
  * is created by the SqlGuiApp instance and contains the document structure with the according methods for manipulation of the document
  * data by SqlGuiView objects. Also, SqlGuiDoc contains the methods for serialization of the document data from and to files.
  *
  * @author Source Framework Automatically Generated by KDevelop, (c) The KDevelop Team. 	
  * @version KDevelop version 1.2 code generation
  */
class SqlGuiDoc : public QObject
{
  Q_OBJECT
  public:
    /** Constructor for the fileclass of the application */
    SqlGuiDoc(QWidget *parent, const char *name=0);
    /** Destructor for the fileclass of the application */
    ~SqlGuiDoc();

    /** database, of the last sql-query */
    QString* resultDB();
    /** database, of the last sql-query */
    QString* resultTable();
    /** list of the column-names of the last sql-query */
    spalten_t* resultCols();

    /** adds a view to the document which represents the document contents. Usually this is your main view. */
    void addView(SqlGuiView *view);

    /** removes a view from the list of currently connected views */
    void removeView(SqlGuiView *view);
    /** sets the modified flag for the document after a modifying action on the view connected to the document.*/
    void setModified(bool _m=true){ modified=_m; };
    /** returns if the document is modified or not. Use this to determine if your document needs saving by the user on closing.*/
    bool isModified(){ return modified; };
    /** "save modified" - asks the user for saving if the document is modified */
    bool saveModified();	
    /** deletes the document's contents */
    void deleteContents();
    /** initializes the document generally */
    bool newDocument();
    /** closes the acutal document */
    void closeDocument();
    /** loads the document by filename and format and emits the updateViews() signal */
    bool openDocument(const QString& url, const QString& uname, const QString& pwd);
    /** saves the document under filename and format.*/	
    bool saveDocument(const KURL& url, const char *format=0);
    /** returns the KURL of the document */
    const KURL& URL() const;
    /** sets the URL of the document */
    void setURL(const KURL& url);
    /** returns the databases */
    base_t * getDBs(){return DB;}
    /** returns the first result-element */
    row_t * getFirstRes(){return RESULT_Start;}
    /** returns the next result-element */
    row_t * getNextRes(){row_t* help=RESULT;RESULT=RESULT->next;if(!RESULT){RESULT=RESULT_Start;} return help;}
		/** returns the last selected table */
    QString * getLastTable();
		/** returns the primary key of the last selected table */
    QString * getPrimaryKey();
	/** returns the hostname */
	QString getHost();
	/** returns the username */
	QString getUser();
	/** returns the password */
	QString getPass();
    /** returns Port */
    unsigned short int getPort();

  public slots:
    /** calls repaint() on all views connected to the document object and is called by the view by which the document has been changed.
     * As this view normally repaints itself, it is excluded from the paintEvent.
     */
    void slotUpdateAllViews(SqlGuiView *sender);
    /** submits the sql-query @Anfrage to the base @Base. */
    void slotDoQuery(QString * Base, QString* Anfrage);
    /** creates a base. */
    void slotDbCreate( QString* );
    /** does a login to the @server with @usernam and @password. */
    void slotDoLogin( QString, QString, QString);
    /** delete the base @db. */
    void slotDbDelete( QString* db );
    /** reconnects to the current server and loads again all bases, tables, columns. */
    void slotReConnect();
    /** deletes the table @tb in the database @db. */
    void slotTableDelete(QString* db, QString* tb);
    /** does a 'CHECK TABLE' on table @tb in base @db. */
    void slotTableCheck(QString* db, QString* tb);
    /** does a 'OPTIMIZE TABLE' on table @tb in base @db. */
    void slotTableOptimize(QString* db, QString* tb);
    /** does a 'REPAIR TABLE' on table @tb in base @db. */
    void slotTableRepair(QString* db, QString* tb);
    /** clears the objekt, that holds all information on the databases, tables, columns. */
    void slotClearMemory();

  public:	
    /** the list of the views currently connected to the document */
    static QList<SqlGuiView> *pViewList;	

 signals:
    void connectionChanged();
    void loginComplete();
    void resultChanged();
    void tableChecked();
    void tableOptimized();
    void tableRepaired();
    void NewDbCreated();
    void NewTableCreated();
    void DbErased();
    void TableErased();
    void DbQueryError( QString* errmsg );
    void queryEmpty();
		void lastActionChanged( QString* db, QString* tab, QString* prim);

  private:
    /** the modified flag of the current document */
    bool modified;
    KURL doc_url;
    QString doc_host;		// Host
	unsigned short int doc_port;			// Port
    QString doc_uname;		// Username
    QString doc_pass;		// Passwort

    base_t * DB;		// DB-Liste
    				// von hier aus bekommt man auch alle Informationen ber Tabellen und Spalten.
    										
    QString *QUERY;	        // die letzte DB-Anfrage
    QString *STATUS;		// DB-Status, Error-Msg. etc
    QString *lastDB;	        // DB der letzten Anfrage
    QString *lastTable;		// Tabelle der letzten Anfrage
    QString *primKey;		// Primary Key der Tabelle der letzten Anfrage

    int stat;			// MySQL-Error-Code

    int field_pos;		// Aktuelle Position in der altuellen Zeile
    int row_pos;		// Aktuelle Zeilen-Position
		
    row_t* RESULT_Start;	// Pointer auf das erste Zeilen-Element der letzten SQL-Anfrage
    row_t* RESULT;	        // Pointer auf das aktuelle Zeilen-Element der letzten SQL-Anfrage

    QString getDbFromSQL(QString * sql);
    QString getTableFromSQL(QString * sql);
    spalten_t * lastCols;       // Pointer auf eine List mit den Spalten-Namen der letzten SQL-Query

    private slots:
    void slotUpdate(MYSQL_RES * res);
    void clearResult();

 private:
    base_t*    cleardb( base_t ** db );
    table_t*   cleartb( table_t ** tb );
    spalten_t* clearsp( spalten_t ** sp );
    row_t*     clearrw( row_t ** rw );
    field_t*   clearfd( field_t ** fd );
};

#endif // SQLGUIDOC_H
